@@ -1,46 +1,50 @@
 `crossbasis` <-
-function(var, group=NULL, vartype="ns", vardf=1, vardegree=1, varknots=NULL,
-	varbound=range(var), varint=FALSE, cen=TRUE, cenvalue=mean(var),
+function(x, vartype="ns", vardf=1, vardegree=1, varknots=NULL,
+	varbound=range(x), varint=FALSE, cen=TRUE, cenvalue=mean(x),
 	maxlag=0, lagtype="ns", lagdf=1, lagdegree=1, lagknots=NULL,
-	lagbound=c(0,maxlag), lagint=TRUE) {
+	lagbound=c(0,maxlag), lagint=TRUE, group=NULL) {
 
 ############################################################################
 # CROSSBASIS 
 #############
 
-basisvar <- mkbasis(var=var,type=vartype,df=vardf,degree=vardegree,
+# CREATE THE BASIS FOR THE PREDICTOR SPACE
+basisvar <- mkbasis(x=x,type=vartype,df=vardf,degree=vardegree,
 	knots=varknots,int=varint,bound=varbound,cen=cen,cenvalue=cenvalue)
 
+# CREATE THE BASIS FOR THE LAG SPACE
 basislag <- mklagbasis(maxlag=maxlag,type=lagtype,df=lagdf,degree=lagdegree,
 	knots=lagknots,int=lagint,bound=lagbound)
 
 # CROSSBASIS COMPUTATION
-basis <- matrix(0,nrow=length(var),ncol=basisvar$df*basislag$df)
-for(v in 1:basisvar$df) {
-	mat <- as.matrix(Lag(basisvar$basis[,v],0:maxlag))
-	for(l in 1:basislag$df) {
+basis <- matrix(0,nrow=length(x),ncol=basisvar$df*basislag$df)
+for(v in seq(length=basisvar$df)) {
+	mat <- as.matrix(Lag(basisvar$basis[, v],0:maxlag))
+	for(l in seq(length=basislag$df)) {
 		basis[,basisvar$df*(l-1)+v] <- mat%*%(basislag$basis[,l])
 	}
 }
-colnames(basis) <- outer(paste("v",1:basisvar$df,sep=""),
-	paste("l",1:basislag$df,sep=""),	function(x,y) paste(x,y,sep="."))
 
-# GROUP 
+# NAMES TO THE NEW CROSS-VARIABLES
+colnames(basis) <- outer(paste("v",seq(length=basisvar$df),sep=""),
+	paste("l",seq(length=basislag$df),sep=""), function(x,y) paste(x,y,sep="."))
+
+# IF GROUP, JUST SET TO NA ALL THE FIRST  
 if(!is.null(group)) {
 	# FIRST COHERENCE CHECKS
 	if(any(is.na(group))) stop("missing values in 'group' are not allowed")
-	if(length(group)!=length(var)) {
-		stop("length(group) must be equal to length(var)")
+	if(length(group)!=length(x)) {
+		stop("length(group) must be equal to length(x)")
 	}
-	if(min(tapply(var,group,length))<=basisvar$df) {
+	if(min(tapply(x,group,length))<=basisvar$df) {
 		stop("each group must have length > vardf")
 	}
-	if(min(tapply(var,group,length))<=maxlag) {
+	if(min(tapply(x,group,length))<=maxlag) {
 		stop("each group must have length > maxlag")
 	}
-# SET TO NA ALL THE FIRST MAXLAG OBS FOR EACH GROUP
-	basis[sequence(tapply(seq(nrow(basis)),
-		group,length))%in%1:maxlag,] <- NA
+	# SET TO NA ALL THE FIRST MAXLAG OBS FOR EACH GROUP
+	basis[sequence(tapply(seq(length=nrow(basis)),
+		group,length))%in%seq(length=maxlag),] <- NA
 }
 
 ############################################################################
@@ -48,8 +52,7 @@ if(!is.null(group)) {
 ############
 
 attributes(basis) <- c(attributes(basis),list(
-	group = length(unique(group)),
-	range = range(var,na.rm=TRUE),
+	range = range(x,na.rm=TRUE),
 	crossdf = basisvar$df*basislag$df,
 
 	vartype = basisvar$type,
@@ -64,12 +67,15 @@ attributes(basis) <- c(attributes(basis),list(
 	maxlag = basislag$maxlag,
 	lagtype = basislag$type,
 	lagdf = basislag$df,
-	lagdegree=basislag$degree,
+	lagdegree = basislag$degree,
 	lagknots = basislag$knots,
 	lagbound = basislag$bound,
 	lagint = basislag$int
 ))
-class(basis) <- "crossbasis"
-basis
+if(!is.null(group)) attributes(basis)$group <- length(unique(group))
+
+class(basis) <- c("crossbasis","matrix")
+
+return(basis)
 }
 
