@@ -1,18 +1,18 @@
 `mkbasis` <-
-function(var, type="ns", df=1, degree=1, knots=NULL, bound=range(var),
-	int=FALSE, cen=TRUE, cenvalue=mean(var)) {
+function(x, type="ns", df=1, degree=1, knots=NULL, bound=range(x),
+	int=FALSE, cen=TRUE, cenvalue=mean(x)) {
+
+# CREATE AN EMPTY LIST
 list <- vector("list",0)
 
-if(!is.null(bound)) {
-	if(any(is.na(bound))) bound <- range(var,na.rm=TRUE)
-}
-if(!is.null(cenvalue)) {
-	if(is.na(cenvalue)) cenvalue <- mean(var,na.rm=TRUE)
-}
+# SET DEFAULT TO BOUND AND CENVALUE IF NA OR NULL
+if(!is.null(bound)&&any(is.na(bound))) bound <- range(x,na.rm=TRUE)
+if(!is.null(cenvalue)&&is.na(cenvalue)) cenvalue <- mean(x,na.rm=TRUE)
 
 ###########################################################################
 # COHERENCE CHECKS
 ##################
+
 # CHANGE IN DOUBLE THRESHOLD
 if(type=="thr") {
 	stop("Type for double threshold changed from 'thr' to 'dthr' since version 0.4.0.
@@ -23,14 +23,14 @@ if(!type%in%c("ns","bs","strata","poly","integer","dthr","lthr","hthr","lin")) {
 	stop("type must be one of ns,bs,strata,poly,integer,dthr,lthr,hthr,lin")
 }
 # CHECK ARGUMENT TYPE
-if(!is.numeric(var)) stop("'var' must be a numeric vector")
-if(!is.numeric(df)|df<1) stop("'df' must be numeric and >=1")
-if(!is.null(knots)&!is.numeric(knots)) stop("'knots' must be numeric")
-if(!is.null(bound)&!is.numeric(bound)) stop("'bound' must be numeric")
+if(!is.numeric(x)) stop("'x' must be a numeric vector")
+if(!is.numeric(df)||df<1) stop("'df' must be numeric and >=1")
+if(!is.null(knots)&&!is.numeric(knots)) stop("'knots' must be numeric")
+if(!is.null(bound)&&!is.numeric(bound)) stop("'bound' must be numeric")
 if(!is.logical(int)) stop("'int' must be logical")
-if(!is.null(cen)&!is.logical(cen)) stop("'cen' must be logical")
-if(!is.null(cenvalue)&!is.numeric(cenvalue)) stop("'cenvalue' must be numeric")
-if(!is.null(degree)) if(!is.numeric(degree)|degree<1)  stop("'degree' must be numeric")
+if(!is.null(cen)&&!is.logical(cen)) stop("'cen' must be logical")
+if(!is.null(cenvalue)&&!is.numeric(cenvalue)) stop("'cenvalue' must be numeric")
+if(!is.null(degree)&&!is.numeric(degree)&&degree<1)  stop("'degree' must be numeric")
 
 # DF FIXED FOR TYPES INTEGER (SOLVED LATER), DTHR AND LIN
 if(type=="poly") df <- degree+int
@@ -47,8 +47,8 @@ if(!is.null(knots)) {
 }
 
 # DF MUST BE <= N. OBS
-if(df+int>length(var)) {
-	stop("df+int must be <= length(var)")
+if(df+int>length(x)) {
+	stop("df+int must be <= length(x)")
 }
 # CENVALUE ONLY WITH CEN=T
 if(cen==F) cenvalue <- NULL
@@ -58,22 +58,20 @@ if(cen==F) cenvalue <- NULL
 #######
 # NS
 #######
-# IF NOT PROVIDED, KNOTS SET TO EQUALLY SPACED QUANTILES
 if(type=="ns")	{
+	# IF NOT PROVIDED, KNOTS SET TO EQUALLY SPACED QUANTILES
 	if(is.null(knots)&df>(1+int)) {
-		knots <- quantile(var,1/(df-int)*1:((df-int)-1),na.rm=TRUE)
+		knots <- quantile(x,1/(df-int)*1:((df-int)-1),na.rm=TRUE)
 	}
+	# IF NOT INTERCEPT-ONLY, CREATE THE BASIS, OTHERWISE A COLUMN OF 1'S
 	if(df>=(1+int)) {
-		if(cen==TRUE) {
-		list$basis <- as.matrix(ns(var,df=df,knots=knots,Bound=bound,
-			int=int)[,] - matrix(ns(cenvalue,df=df,knots=knots,
-			Bound=bound,int=int)[,], 
-			nrow=length(var),ncol=length(knots)+1+int,byrow=TRUE))
-		} else {
-			list$basis <- as.matrix(ns(var,df=df,knots=knots,
+		list$basis <- as.matrix(ns(x,df=df,knots=knots,
 			Bound=bound,int=int)[,])
+		if(cen==TRUE) {
+			list$basis <- sweep(list$basis,2,ns(cenvalue,df=df,
+				knots=knots,Bound=bound,int=int)[,],"-")
 		}
-	} else list$basis <- as.matrix(rep(1,length(var)))
+	} else list$basis <- as.matrix(rep(1,length(x)))
 	degree <- NULL
 }
 
@@ -84,21 +82,18 @@ if(type=="ns")	{
 if(type=="bs")	{
 	if(df<degree+int) stop("df must be >=degree+int for type='bs'")
 	if(is.null(knots)&df>(degree+int)) {
-		knots <- quantile(var,1/(df-int-degree+1)*1:((df-int)-degree),
+		knots <- quantile(x,1/(df-int-degree+1)*1:((df-int)-degree),
 			na.rm=TRUE)
 	}
+	# IF NOT INTERCEPT-ONLY, CREATE THE BASIS, OTHERWISE A COLUMN OF 1'S
 	if(df>=(1+int)) {
+		list$basis <- as.matrix(bs(x,df=df,degree=degree,knots=knots,
+			Bound=bound,int=int)[,])
 		if(cen==TRUE) {
-		list$basis <- as.matrix(bs(var,df=df,knots=knots,Bound=bound,
-			int=int,degree=degree)[,] - matrix(bs(cenvalue,df=df,
-			knots=knots,Bound=bound,int=int,degree=degree)[,], 
-			nrow=length(var),ncol=length(knots)+degree+int,byrow=TRUE))
-		} else {
-			list$basis <- as.matrix(bs(var,df=df,knots=knots,
-			Bound=bound,int=int,degree=degree)[,])
+			list$basis <- sweep(list$basis,2,bs(cenvalue,df=df,
+				degree=degree,knots=knots,Bound=bound,int=int)[,],"-")
 		}
-	} else list$basis <- as.matrix(rep(1,length(var)))
-
+	} else list$basis <- as.matrix(rep(1,length(x)))
 }
 
 ##########
@@ -108,12 +103,15 @@ if(type=="bs")	{
 # IF ONLY DF PROVIDED, KNOTS PLACED AT EQUALLY SPACED QUANTILES
 # KNOTS SPECIFYING EMPTY COLUMNS (AND THE COLUMNS THEMSELVES) ARE EXCLUDED
 if(type=="strata")	{
-	range <- range(var,na.rm=TRUE)
+	range <- range(x,na.rm=TRUE)
+	# IF NOT PROVIDED, KNOTS SET TO EQUALLY SPACED QUANTILES
 	if(is.null(knots)&df>1) {
-		knots <- quantile(var,1/(df+1-int)*1:(df-int),na.rm=TRUE)
+		knots <- quantile(x,1/(df+1-int)*1:(df-int),na.rm=TRUE)
 	}
-	x <- cut(var,c(range[1],knots,range[2]+0.1),right=FALSE)
+	# CREATE A DESIGN MATRIX WITH DUMMY VARIABLES
+	x <- cut(x,c(range[1],knots,range[2]+0.1),right=FALSE)
 	list$basis <- as.matrix(outer(x,levels(x),"==")+0)
+	# IF WITH INTERCEPT AND MORE THAN 1 COLUMN, ELIMINATE THE FIRST COLUMN
 	if(int==FALSE & !is.null(knots)) {
 		list$basis <- as.matrix(list$basis[,-1])
 	}
@@ -129,8 +127,8 @@ if(type=="strata")	{
 ########
 # THE NUMBER OF KNOTS CAN BE USED TO SPECIFY THE DF (NKNOTS+1+INT)
 if(type=="poly")	{
-	if(cen==TRUE) list$basis <- outer(var-(cenvalue),(1-int):(df-int),"^")
-	if(cen==FALSE) list$basis <- outer(var,(1-int):(df-int),"^")
+	if(cen==TRUE) list$basis <- outer(x-(cenvalue),(1-int):(df-int),"^")
+	if(cen==FALSE) list$basis <- outer(x,(1-int):(df-int),"^")
 	knots <- NULL
 	bound <- NULL
 }
@@ -140,7 +138,7 @@ if(type=="poly")	{
 ###########
 # NEVER CENTERED, IT DOESN'T TAKE INTO ACCOUNT DF OR KNOTS
 if(type=="integer")	{
-	x <- factor(round(var,0))
+	x <- factor(round(x,0))
 	list$basis <- as.matrix(outer(x,levels(x),"==")+0)
 	if(int==FALSE&length(levels(x))>1) list$basis <- list$basis[,-1]
 	df <- ncol(list$basis)
@@ -158,11 +156,11 @@ if(type=="integer")	{
 # ONLY THE FIRST AND LAST KNOTS CONSIDERED, KNOT REPLIED IF SINGLE (VMODEL)
 if(type=="dthr")	{
 	if(is.null(knots)) {
-		knots <- quantile(var,(1/3)*1:2,na.rm=TRUE)
+		knots <- quantile(x,(1/3)*1:2,na.rm=TRUE)
 	}
 	knots <- c(knots[1],knots[length(knots)])
-	list$basis <- as.matrix(cbind(-pmin(var-knots[1],0),
-		pmax(var-knots[2],0)))
+	list$basis <- as.matrix(cbind(-pmin(x-knots[1],0),
+		pmax(x-knots[2],0)))
 	if(int==TRUE) list$basis <- cbind(1,list$basis)
 	bound <- NULL
 	cen <- FALSE
@@ -177,9 +175,9 @@ if(type=="dthr")	{
 # IF ONLY DF PROVIDED, KNOTS PLACED AT EQUALLY SPACED QUANTILES
 if(type=="lthr")	{
 	if(is.null(knots)) {
-		knots <- quantile(var,1/(df+1-int)*1:(df-int),na.rm=TRUE)
+		knots <- quantile(x,1/(df+1-int)*1:(df-int),na.rm=TRUE)
 	}
-	list$basis <- as.matrix(outer(var,knots, function(a,b) -pmin(a-b,0)))
+	list$basis <- as.matrix(outer(x,knots, function(a,b) -pmin(a-b,0)))
 	if(int==TRUE) list$basis <- cbind(1,list$basis)
 	bound <- NULL
 	cen <- FALSE
@@ -194,9 +192,9 @@ if(type=="lthr")	{
 # IF ONLY DF PROVIDED, KNOTS PLACED AT EQUALLY SPACED QUANTILES
 if(type=="hthr")	{
 	if(is.null(knots)) {
-		knots <- quantile(var,1/(df+1-int)*1:(df-int),na.rm=TRUE)
+		knots <- quantile(x,1/(df+1-int)*1:(df-int),na.rm=TRUE)
 	}
-	list$basis <- as.matrix(outer(var,knots, function(a,b) pmax(a-b,0)))
+	list$basis <- as.matrix(outer(x,knots, function(a,b) pmax(a-b,0)))
 	if(int==TRUE) list$basis <- cbind(1,list$basis)
 	bound <- NULL
 	cen <- FALSE
@@ -209,8 +207,8 @@ if(type=="hthr")	{
 #######
 # IT DOESN'T TAKE INTO ACCOUNT DF AND KNOTS
 if(type=="lin")	{
-	if(cen==TRUE) list$basis <- as.matrix(var-cenvalue)
-	if(cen==FALSE) list$basis <- as.matrix(var)
+	if(cen==TRUE) list$basis <- as.matrix(x-cenvalue)
+	if(cen==FALSE) list$basis <- as.matrix(x)
 	if(int==TRUE) list$basis <- cbind(1,list$basis)
 	knots <- NULL
 	bound <- NULL
@@ -220,6 +218,7 @@ if(type=="lin")	{
 ##########################################################################
 
 colnames(list$basis) <- paste("b",1:df,sep="")
+
 list$type <- type
 list$df <- df
 list$degree <- degree
@@ -228,6 +227,7 @@ list$bound <- bound
 list$int <- int
 list$cen <- cen
 list$cenvalue <- cenvalue
-list
+
+return(list)
 }
 
